package com.gic.haoban.manage.api.service.licence;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.haoban.manage.api.dto.licence.LicenceOrderDTO;
import com.gic.haoban.manage.api.dto.licence.LicenceOrderPageDTO;
import com.gic.haoban.manage.api.dto.qdto.licence.LicenceOrderPageQDTO;
import com.gic.haoban.manage.api.dto.qdto.licence.LicenceOrderQDTO;

/**
 * <p>
 * 服务类
 * </p>
 *
 * @author jx
 * @since 2023-03-09
 */
public interface LicenceOrderApiService {

    /**
     * 获取订单详情
     *
     * @param orderId 订单id
     * @return LicenceOrderDTO
     */
    ServiceResponse<LicenceOrderDTO> getLicenceOrderDetail(Long orderId,Integer type);

    /**
     * 删除订单
     *
     * @param orderId 订单id
     * @return
     */
    ServiceResponse<Boolean> deleteLicenceOrder(Long orderId);

    /**
     * 新增或者修改订单详情
     *
     * @param licenceOrderQDTO
     * @return
     */
    ServiceResponse<String> saveOrUpdateLicenceOrder(LicenceOrderQDTO licenceOrderQDTO);

    /**
     * 分页查询订单数据
     *
     * @param licenceOrderPageQDTO
     * @return
     */
    ServiceResponse<Page<LicenceOrderPageDTO>> getLicenceOrderPage(LicenceOrderPageQDTO licenceOrderPageQDTO);

    /**
     * 更新订单状态
     *
     * @param orderId 订单id
     * @param type    订单状态
     * @return
     */
    ServiceResponse<Boolean> updateLicenceOrderType(Long orderId, Integer type,String enterpriseId,String wxEnterpriseId,String creatorName);

    /**
     * 订单凭证上传
     *
     * @param orderId 订单id
     * @param voucher 凭证url
     * @return
     */
    ServiceResponse<Boolean> uploadLicenceOrderVoucher(Long orderId, String voucher,String enterpriseId,String wxEnterpriseId,String creatorName);

    /**
     * 订单支付接口
     *
     * @param orderId 订单id
     * @return
     */
    ServiceResponse<Boolean> payLicenceOrder(Long orderId);

    /**
     * 查询企业是否存在订单未支付接口
     *
     * @param wxEnterpriseId 企业微信id
     * @return
     */
    ServiceResponse<Boolean> isPayLicenceOrder(String wxEnterpriseId);

    /**
     * 微信通知回调接口
     */
    ServiceResponse<Boolean> wxCallBack(String params);
    /**
     * 企业微信通知回调接口
     */
    ServiceResponse<Boolean> qywxCallBack(String params);


}
