package com.gic.haoban.manage.service.service.licence.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.enterprise.api.dto.EnterpriseDTO;
import com.gic.enterprise.api.service.EnterpriseService;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.dto.licence.LicenceOrderPageDTO;
import com.gic.haoban.manage.api.dto.qdto.licence.LicenceOrderPageQDTO;
import com.gic.haoban.manage.api.dto.qdto.licence.LicenceOrderQDTO;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.service.dao.mapper.licence.TabHaobanLicenceOrderMapper;
import com.gic.haoban.manage.service.dao.mapper.licence.TabHaobanLicenceOrderProgressMapper;
import com.gic.haoban.manage.service.entity.licence.TabHaobanLicenceOrder;
import com.gic.haoban.manage.service.entity.licence.TabHaobanLicenceOrderProgress;
import com.gic.haoban.manage.service.service.licence.LicenceOrderService;
import com.gic.haoban.manage.service.util.DingUtils;
import com.github.pagehelper.PageHelper;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * @description:
 * @Author: wenhua
 * @Date: 2023/3/9 17:01
 */
public class LicenceOrderServiceImpl implements LicenceOrderService {

    @Autowired
    private TabHaobanLicenceOrderMapper tabHaobanLicenceOrderMapper;
    @Autowired
    private TabHaobanLicenceOrderProgressMapper tabHaobanLicenceOrderProgressMapper;
    @Autowired
    private EnterpriseService enterpriseService;

    @Autowired
    private WxEnterpriseApiService wxEnterpriseApiService;

    @Override
    public TabHaobanLicenceOrder getLicenceOrderDetail(Long orderId) {
        TabHaobanLicenceOrder tabHaobanLicenceOrder = tabHaobanLicenceOrderMapper.selectById(orderId);
        return tabHaobanLicenceOrder;
    }

    @Override
    public Integer deleteLicenceOrder(Long orderId) {
        return tabHaobanLicenceOrderMapper.deleteById(orderId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String saveOrUpdateLicenceOrder(LicenceOrderQDTO licenceOrderQDTO) {
        long orderId = UniqueIdUtils.uniqueLong();
        if(ObjectUtil.isNull(licenceOrderQDTO.getOrderId())){
            //订单不支持修改，只能新增
            TabHaobanLicenceOrder order = JSONObject.parseObject(JSONObject.toJSONString(licenceOrderQDTO), TabHaobanLicenceOrder.class);
            Date now = new Date();
            order.setOrderId(orderId);
            order.setCreateTime(now);
            order.setUpdateTime(now);
            Date expireTime;
            if(order.getPayType()==1){
                expireTime = DateUtil.addNumForMinute(now, 30);
            }else {
                //生成订单编号
                String num = "ZH"+UniqueIdUtils.uniqueLong();
                order.setTransactionId(num);
                expireTime = DateUtil.addDay(now, 3);
            }
            order.setExpireTime(expireTime);
            tabHaobanLicenceOrderMapper.insert(order);
            //记录订单进度
            saveOrderProgress(orderId,"创建订单",licenceOrderQDTO.getEnterpriseId(),licenceOrderQDTO.getWxEnterpriseId(),licenceOrderQDTO.getCreatorName());
        }
        return String.valueOf(orderId);
    }

    private void saveOrderProgress(Long orderId,String orderStatus,String enterpriseId,String wxEnterpriseId,String creatorName){
        TabHaobanLicenceOrderProgress orderProgress = new TabHaobanLicenceOrderProgress();
        orderProgress.setId(UniqueIdUtils.uniqueLong());
        orderProgress.setEnterpriseId(enterpriseId);
        orderProgress.setWxEnterpriseId(wxEnterpriseId);
        orderProgress.setOrderId(orderId);
        orderProgress.setOrderStatus(orderStatus);
        orderProgress.setType(1);
        orderProgress.setCreatorName(creatorName);
        tabHaobanLicenceOrderProgressMapper.insert(orderProgress);
    }
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean updateLicenceOrderType(Long orderId, Integer type,String enterpriseId,String wxEnterpriseId,String creatorName) {
        tabHaobanLicenceOrderMapper.updateLicenceOrderType(orderId,type);
        //记录订单进度
        switch (type){
            case 0:
                saveOrderProgress(orderId,"创建订单",enterpriseId,wxEnterpriseId,creatorName);
                break;
            case 1:
                saveOrderProgress(orderId,"已支付",enterpriseId,wxEnterpriseId,creatorName);
                break;
            case 2:
                saveOrderProgress(orderId,"已取消",enterpriseId,wxEnterpriseId,creatorName);
                break;
            case 3:
                saveOrderProgress(orderId,"已过期",enterpriseId,wxEnterpriseId,creatorName);
                break;
            case 4:
                saveOrderProgress(orderId,"已退款",enterpriseId,wxEnterpriseId,creatorName);
                break;
            case 5:
                saveOrderProgress(orderId,"凭证已上传，审核中",enterpriseId,wxEnterpriseId,creatorName);
                break;
            default:
                break;
        }
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean uploadLicenceOrderVoucher(Long orderId, String voucher,String enterpriseId,String wxEnterpriseId,String creatorName) {
        TabHaobanLicenceOrder tabHaobanLicenceOrder = tabHaobanLicenceOrderMapper.selectById(orderId);
        if (tabHaobanLicenceOrder == null){
            return  false;
        }
        tabHaobanLicenceOrderMapper.uploadLicenceOrderVoucher(orderId,voucher);
        //记录订单进度
        if (StrUtil.isNotBlank(tabHaobanLicenceOrder.getVoucher())){
            saveOrderProgress(orderId,"凭证已重新上传，审核中",enterpriseId,wxEnterpriseId,creatorName);
        }else {
            saveOrderProgress(orderId,"凭证已上传，审核中",enterpriseId,wxEnterpriseId,creatorName);
        }
        //查询gic品牌名称
        String enterpriseName="";
        String wxEnterpriseName="";
        EnterpriseDTO enterpriseDTO = enterpriseService.getEnterpriseById(enterpriseId);
        if (ObjectUtil.isNotNull(enterpriseDTO)){
            enterpriseDTO.getEnterpriseName();
        }
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseApiService.getOne(wxEnterpriseId);
        if (ObjectUtil.isNotNull(wxEnterpriseDTO)){
            wxEnterpriseDTO.getCorpName();
        }
        String dateToStr = DateUtil.dateToStr(new Date(), DateUtil.FORMAT_DATETIME_19);
        //发送钉钉消息通知
        String msg = "标题：企微账号许可通知 品牌名称:"+enterpriseName+" 企业名称："+wxEnterpriseName+" 消息内容："+dateToStr+"新增订单"+orderId+",请尽快处理";
        DingUtils.send(msg,"https://oapi.dingtalk.com/robot/send?access_token=63074c9fc1ac2c00aa62477facdf16e5fa2388cf6a621edba1fb05d6e55d031f",false);
        return true;
    }

    @Override
    public String isPayLicenceOrder(String wxEnterpriseId) {
        String orderId = tabHaobanLicenceOrderMapper.selectByWxEnterpriseId(wxEnterpriseId);
        if (StrUtil.isNotBlank(orderId)){
            return orderId;
        }
        return null;
    }

    @Override
    public Page<LicenceOrderPageDTO> getLicenceOrderPage(LicenceOrderPageQDTO qdto) {
        //时间处理
        if (StringUtils.isNotBlank(qdto.getStartTime())) {
            qdto.setStartTime(qdto.getStartTime() + " 00:00:00");
        }
        if (StringUtils.isNotBlank(qdto.getEndTime())) {
            qdto.setEndTime(qdto.getEndTime() + " 23:59:59");
        }
        PageHelper.startPage(qdto);
        List<LicenceOrderPageDTO> licenceOrderPage = tabHaobanLicenceOrderMapper.getLicenceOrderPage(qdto);
        //判断订单状态，支付时间已过的订单的需要更改订单状态为已过期
        for (LicenceOrderPageDTO dto : licenceOrderPage){
            if (dto.getExpireTime().before(new Date())){
                dto.setOrderStatus(3);
                updateLicenceOrderType(dto.getOrderId(), 3,qdto.getEnterpriseId(), qdto.getWxEnterpriseId(), qdto.getCreatorName());
            }
        }
        return PageHelperUtils.changePageHelperToCurrentPage(licenceOrderPage, LicenceOrderPageDTO.class);

    }
}
