package com.gic.haoban.manage.service.service.out.impl.licence;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.api.dto.EnterpriseDTO;
import com.gic.enterprise.api.service.EnterpriseService;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.dto.licence.LicenceOrderDTO;
import com.gic.haoban.manage.api.dto.licence.LicenceOrderPageDTO;
import com.gic.haoban.manage.api.dto.licence.LicenceOrderProgressDTO;
import com.gic.haoban.manage.api.dto.qdto.licence.LicenceOrderPageQDTO;
import com.gic.haoban.manage.api.dto.qdto.licence.LicenceOrderQDTO;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.api.service.licence.LicenceOrderApiService;
import com.gic.haoban.manage.service.entity.licence.TabHaobanLicenceOrder;
import com.gic.haoban.manage.service.entity.licence.TabHaobanLicenceOrderProgress;
import com.gic.haoban.manage.service.service.licence.LicenceOrderProgressService;
import com.gic.haoban.manage.service.service.licence.LicenceOrderService;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @description:
 * @Author: wenhua
 * @Date: 2023/3/9 17:04
 */
public class LicenceOrderApiServiceImpl implements LicenceOrderApiService {

    private static final Logger logger = LogManager.getLogger(LicenceOrderApiServiceImpl.class);

    @Autowired
    private LicenceOrderService licenceOrderService;
    @Autowired
    private LicenceOrderProgressService licenceOrderProgressService;

    @Autowired
    private EnterpriseService enterpriseService;

    @Autowired
    private WxEnterpriseApiService wxEnterpriseApiService;

    @Override
    public ServiceResponse<LicenceOrderDTO> getLicenceOrderDetail(Long orderId,Integer type) {
        TabHaobanLicenceOrder licenceOrderDetail = licenceOrderService.getLicenceOrderDetail(orderId);
        if (licenceOrderDetail != null){
            //查询订单进度表
            LicenceOrderDTO licenceOrderDTO = JSONObject.parseObject(JSONObject.toJSONString(licenceOrderDetail), LicenceOrderDTO.class);
            List<TabHaobanLicenceOrderProgress> list = licenceOrderProgressService.getListByOrderId(orderId,type);
            licenceOrderDTO.setOrderProgressList(EntityUtil.changeEntityListByJSON(LicenceOrderProgressDTO.class, list));
            //查询gic品牌名称
            EnterpriseDTO enterpriseDTO = enterpriseService.getEnterpriseById(licenceOrderDTO.getEnterpriseId());
            if (ObjectUtil.isNotNull(enterpriseDTO)){
                licenceOrderDTO.setEnterpriseName(enterpriseDTO.getEnterpriseName());
            }
            WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseApiService.getOne(licenceOrderDTO.getWxEnterpriseId());
            if (ObjectUtil.isNotNull(wxEnterpriseDTO)){
                logger.info("企业微信查询返回实体：{}", JSON.toJSONString(wxEnterpriseDTO));
                licenceOrderDTO.setWxEnterpriseName(wxEnterpriseDTO.getCorpName());
            }
            return ServiceResponse.success(licenceOrderDTO);
        }
        return ServiceResponse.success();
    }

    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public ServiceResponse<Boolean> deleteLicenceOrder(Long orderId) {
        licenceOrderService.deleteLicenceOrder(orderId);
        licenceOrderProgressService.deleteByOrderId(orderId);
        return ServiceResponse.success(true);
    }

    @Override
    public ServiceResponse<String> saveOrUpdateLicenceOrder(LicenceOrderQDTO licenceOrderQDTO) {
        String s = licenceOrderService.saveOrUpdateLicenceOrder(licenceOrderQDTO);
        return ServiceResponse.success(s);
    }

    @Override
    public ServiceResponse<Page<LicenceOrderPageDTO>> getLicenceOrderPage(LicenceOrderPageQDTO licenceOrderPageQDTO) {
        Page<LicenceOrderPageDTO> licenceOrderPage = licenceOrderService.getLicenceOrderPage(licenceOrderPageQDTO);
        return ServiceResponse.success(licenceOrderPage);
    }

    @Override
    public ServiceResponse<Boolean> updateLicenceOrderType(Long orderId, Integer type,String enterpriseId,String wxEnterpriseId,String creatorName) {
        licenceOrderService.updateLicenceOrderType(orderId,type,enterpriseId,wxEnterpriseId,creatorName);
        return ServiceResponse.success(true);
    }

    @Override
    public ServiceResponse<Boolean> uploadLicenceOrderVoucher(Long orderId, String voucher,String enterpriseId,String wxEnterpriseId,String creatorName) {
        licenceOrderService.uploadLicenceOrderVoucher(orderId,voucher,enterpriseId,wxEnterpriseId,creatorName);
        return ServiceResponse.success(true);
    }

    @Override
    public ServiceResponse<Boolean> payLicenceOrder(Long orderId) {

        return null;
    }

    @Override
    public ServiceResponse<Boolean> isPayLicenceOrder(String wxEnterpriseId) {
        String order = licenceOrderService.isPayLicenceOrder(wxEnterpriseId);
        if (StrUtil.isNotBlank(order)){
            return ServiceResponse.success(true);
        }
        return ServiceResponse.success(false);
    }

    @Override
    public ServiceResponse<Boolean> wxCallBack(String params) {
        logger.info("接收微信回调参数：{}",params);


        return null;
    }

    @Override
    public ServiceResponse<Boolean> qywxCallBack(String params) {
        logger.info("接收企业微信回调参数：{}",params);


        return null;
    }
}
